// libraries import
#include <Adafruit_NeoPixel.h>

// giving a name to the pins of different sensors and actuators
#define PRESS 2
#define BUTTON 3
#define RING_PIN 6
#define PIXEL_COUNT 12

// Initialising the NeoPixel Ring with the amount of pixels(LEDs), the pin it's on, and flags for the type and frequency of the bitstream
Adafruit_NeoPixel ring = Adafruit_NeoPixel(PIXEL_COUNT, RING_PIN, NEO_GRB + NEO_KHZ800);

// oldState is used to keep track of the state of the button
bool oldState = HIGH;
int effect = 0;

void setup() {
  pinMode(PRESS, INPUT_PULLUP);
  pinMode(BUTTON, INPUT_PULLUP);
  // initialize pixels to off
  ring.begin();
  ring.show();
}

void loop() {
  // read pressure sensor, and turn off/set effect to 0 if it's not pressed
  bool sensor = digitalRead(PRESS);
  if (sensor == HIGH) {
    effect = 0;
    startEffect(effect);
  }

  // check if the button has been pressed, wasn't pressed last time it checked and if pressure sensor is pressed
  bool newState = digitalRead(BUTTON);
  if (newState == LOW && oldState == HIGH && sensor == LOW) {
    // cycle to next effect and start it
    effect++;
    if (effect > 11) {
      effect = 0;
    }
    // run the effect
    startEffect(effect);
  }
  oldState = newState;
  delay(250);
}

// switch case statement to handle all the different effects
void startEffect(int effect) {
  switch (effect) {
    case 0: colorWipe(ring.Color(0, 0, 0), 100);         // Off
      break;
    case 1: colorWipe(ring.Color(0, 50, 0), 100);        // Green
      break;
    case 2: colorWipe(ring.Color(50, 40, 0), 100);       // Yellow
      break;
    case 3: colorWipe(ring.Color(50, 14, 0), 100);       // Orange
      break;
    case 4: colorWipe(ring.Color(50, 0, 0), 100);        // Red
      break;
    case 5: colorWipe(ring.Color(50, 0, 15), 100);       // Pink
      break;
    case 6: colorWipe(ring.Color(20, 0, 50), 100);       // Purple
      break;
    case 7: colorWipe(ring.Color(0, 0, 50), 100);        // Blue
      break;
    case 8: theaterChase(ring.Color(50, 0, 0), 70);     // Red chase
      break;
    case 9: theaterChase(ring.Color(0, 50, 0), 70);     // Blue chase
      break;
    case 10: theaterChase(ring.Color(0, 0, 50), 70);    // Green chase
      break;
    case 11: rainbow(20);
      break;
  }
}

// loops past all pixels and lights them one by one with a delay
void colorWipe(uint32_t c, uint8_t wait) {
  for (uint16_t i = 0; i < ring.numPixels(); i++) {
    ring.setPixelColor(i, c);
    ring.show();
    delay(wait);
  }
}

// effect where 4 out of 12 pixels are lit, and every cycle the next 4 are lit, and the current 4 are turned off again, giving a circular chasing effect like in theater lights
void theaterChase(uint32_t c, uint8_t wait) {
  for (int j = 0; j < 10; j++) {  //do 10 cycles of chasing
    for (int q = 0; q < 3; q++) {
      for (int i = 0; i < ring.numPixels(); i = i + 3) {
        ring.setPixelColor(i + q, c);    //turn every third pixel on
      }
      ring.show();

      delay(wait);
      for (int i = 0; i < ring.numPixels(); i = i + 3) {
        ring.setPixelColor(i + q, 0);      //turn every third pixel off
      }
    }
  }
}

// rainbow fade that slowly goes from red to blue to green and back to red, using the Wheel function
void rainbow(int wait) {
  uint16_t i, j;

  for (j = 0; j < 256; j++) {
    for (i = 0; i < ring.numPixels(); i++) {
      ring.setPixelColor(i, Wheel((j)));
    }
    ring.show();
    delay(wait);
  }
}

// Starts at red, adding green and removing red until fully green at 85, then adding blue and removing green until fully blue at 170, 
// then adding red and removing blue until fully red again at 255
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if (WheelPos < 85) {
    return ring.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if (WheelPos < 170) {
    WheelPos -= 85;
    return ring.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return ring.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}
